float t = 0.0; // 投射されてからの時間
float v0 = 0.0; // 初速
float theta = 0.0; // 投射角度
float angle = 45.0; // 砲台の角度
float bx = 0.0, by = 0.0; // ボールの座標
float hitX = 500, hitY = 300; // ターゲットの座標
int hit = 0; // ターゲットにあたったかどうか

void setup() {
  // 600x400のウインドウを作成する
  size(600, 400);
  // 文字のを大きさをある程度大きく
  textSize( 30 );
}

void draw() {
  // 背景はとりあえず白く塗りつぶそう
  background(255);
  // 線の太さをある程度太くしておく
  strokeWeight(5);
  // 砲台を描画する
  line( 0, height, 30*cos(radians(angle)), height-30*sin(radians(angle)) );

  // 初速と角度を利用して投射
  // ターゲットにあたっている場合（hit==1）は赤色で描画
  // また、ボールは移動しない。一方、ターゲットにあたっていない
  // 場合（hit==0）は、斜方投射の公式でt秒後の座標(bx, by)を計算
  if ( hit == 0 ) {
    fill( 255, 0, 0 );

    // 注意するのはthetaは度なのでラジアンに変換してあげるということ
    bx = v0*t*cos( radians(theta) );
    by = v0*t*sin( radians(theta) )-0.5*9.8*t*t;
    // Y軸は逆なので高さで座標を補正する
    by = height - by;

    // 右端まで来ていない場合はそのまま表示  
    // 右端を超えた場合は，xの座標を補正
    // bx = width-(bx-width)
    if ( bx >= width ) {
      bx = width*2-bx;
    }

    // ボールがターゲットの中に入っているかどうかは
    // ボールの左上座標(bx-15, by-15)、右下座標(bx+15, by+15)が
    // 四角形の中に入っているかどうかで判定する
    if ( bx-15 >= hitX && bx+15 <= hitX+50 && by-15 >= hitY && by+15 <= hitY+50 ) {
      // 入っていればhit=1とする
      hit = 1;
    }
  }

  // hit==0の場合は当たっていないので白色で，hit==1の場合は赤色で描画
  if ( hit == 0 ) {
    fill( 255 );
  } else {
    fill( 255, 0, 0 );
  }

  // ターゲットを描画する
  rect( hitX, hitY, 50, 50 );
  // ボールを描画する
  ellipse( bx, by, 30, 30 );

  // tを増やす
  t = t + 0.1;
  // 砲台の角度を表示する
  fill( 0 );
  text( angle+" degrees", 30, 30 );
}

void keyPressed() {
  // キーボードのボタンがおされたら色々動作を変更する
  if ( keyCode == UP ) {
    // 上が押されたら角度上昇
    angle += 1.0;
  } else if ( keyCode == DOWN ) {
    // 上が押されたら角度下降
    angle -= 1.0;
  } else if ( keyCode == RIGHT ) {
    // 右が押されたら発射
    t = 0.0; // tを0に変更
    theta = angle; // 投射角度を砲台の角度にセット
    v0 = 100; // ずっと100なのだけど一応
  }
}

